@@ -3,10 +3,18 @@
 // Copyright (C) 2018-2020 The Nitrocli Developers
 // SPDX-License-Identifier: GPL-3.0-or-later
 
+use std::borrow;
 use std::convert::TryFrom as _;
+use std::env;
+use std::ffi;
 use std::fmt;
+use std::fs;
+use std::io;
 use std::mem;
+use std::ops;
 use std::ops::Deref as _;
+use std::path;
+use std::process;
 use std::thread;
 use std::time;
 use std::u8;
@@ -22,9 +30,12 @@ use nitrokey::GetPasswordSafe;
 
 use crate::args;
 use crate::config;
+use crate::output;
 use crate::pinentry;
 use crate::Context;
 
+const NITROCLI_EXT_PREFIX: &str = "nitrocli-";
+
 /// Set `libnitrokey`'s log level based on the execution context's verbosity.
 fn set_log_level(ctx: &mut Context<'_>) {
   let log_lvl = match ctx.config.verbosity {
@@ -334,11 +345,13 @@ where
 fn print_storage_status(
   ctx: &mut Context<'_>,
   status: &nitrokey::StorageStatus,
+  sd_card_usage: &ops::Range<u8>,
 ) -> anyhow::Result<()> {
   println!(
     ctx,
     r#"  Storage:
     SD card ID:        {id:#x}
+    SD card usage:     {usagestart}% .. {usageend}% not written
     firmware:          {fw}
     storage keys:      {sk}
     volumes:
@@ -346,6 +359,8 @@ fn print_storage_status(
       encrypted:       {ve}
       hidden:          {vh}"#,
     id = status.serial_number_sd_card,
+    usagestart = sd_card_usage.start,
+    usageend = sd_card_usage.end,
     fw = if status.firmware_locked {
       "locked"
     } else {
@@ -364,11 +379,7 @@ fn print_storage_status(
 }
 
 /// Query and pretty print the status that is common to all Nitrokey devices.
-fn print_status(
-  ctx: &mut Context<'_>,
-  model: &'static str,
-  device: &nitrokey::DeviceWrapper<'_>,
-) -> anyhow::Result<()> {
+fn print_status(ctx: &mut Context<'_>, device: &nitrokey::DeviceWrapper<'_>) -> anyhow::Result<()> {
   let serial_number = device
     .get_serial_number()
     .context("Could not query the serial number")?;
@@ -381,7 +392,7 @@ fn print_status(
   firmware version:  {fwv}
   user retry count:  {urc}
   admin retry count: {arc}"#,
-    model = model,
+    model = device.get_model(),
     id = serial_number,
     fwv = device
       .get_firmware_version()
@@ -398,8 +409,11 @@ fn print_status(
     let status = device
       .get_storage_status()
       .context("Failed to retrieve storage status")?;
+    let sd_card_usage = device
+      .get_sd_card_usage()
+      .context("Failed to retrieve SD card usage")?;
 
-    print_storage_status(ctx, &status)
+    print_storage_status(ctx, &status, &sd_card_usage)
   } else {
     Ok(())
   }
@@ -407,13 +421,7 @@ fn print_status(
 
 /// Inquire the status of the nitrokey.
 pub fn status(ctx: &mut Context<'_>) -> anyhow::Result<()> {
-  with_device(ctx, |ctx, device| {
-    let model = match device {
-      nitrokey::DeviceWrapper::Pro(_) => "Pro",
-      nitrokey::DeviceWrapper::Storage(_) => "Storage",
-    };
-    print_status(ctx, model, &device)
-  })
+  with_device(ctx, |ctx, device| print_status(ctx, &device))
 }
 
 /// List the attached Nitrokey devices.
@@ -461,6 +469,50 @@ pub fn list(ctx: &mut Context<'_>, no_connect: bool) -> anyhow::Result<()> {
   Ok(())
 }
 
+/// Fill the SD card with random data
+pub fn fill(ctx: &mut Context<'_>, attach: bool) -> anyhow::Result<()> {
+  with_storage_device(ctx, |ctx, mut device| {
+    let mut initial_progress = 0;
+    if attach {
+      let status = device
+        .get_operation_status()
+        .context("Failed to query operation status")?;
+      match status {
+        nitrokey::OperationStatus::Ongoing(progress) => initial_progress = progress,
+        nitrokey::OperationStatus::Idle => anyhow::bail!("No fill operation in progress"),
+      }
+    } else {
+      let pin_entry = pinentry::PinEntry::from(args::PinType::Admin, &device)?;
+
+      // Similar to reset, we want the user to re-enter the admin PIN
+      // even if is cached to avoid accidental data loss.
+      pinentry::clear(&pin_entry).context("Failed to clear cached secret")?;
+
+      try_with_pin(ctx, &pin_entry, |pin| {
+        device.fill_sd_card(&pin).context("Failed to fill SD card")
+      })?;
+    }
+
+    let mut progress_bar = output::ProgressBar::new(initial_progress);
+    progress_bar.draw(ctx)?;
+
+    while !progress_bar.is_finished() {
+      thread::sleep(time::Duration::from_secs(1));
+
+      let status = device
+        .get_operation_status()
+        .context("Failed to query operation status")?;
+      match status {
+        nitrokey::OperationStatus::Ongoing(progress) => progress_bar.update(progress)?,
+        nitrokey::OperationStatus::Idle => progress_bar.finish(),
+      };
+      progress_bar.draw(ctx)?;
+    }
+
+    Ok(())
+  })
+}
+
 /// Perform a factory reset.
 pub fn reset(ctx: &mut Context<'_>) -> anyhow::Result<()> {
   with_device(ctx, |ctx, mut device| {
@@ -614,13 +666,13 @@ pub fn config_get(ctx: &mut Context<'_>) -> anyhow::Result<()> {
     println!(
       ctx,
       r#"Config:
-  numlock binding:          {nl}
-  capslock binding:         {cl}
-  scrollock binding:        {sl}
+  num lock binding:         {nl}
+  caps lock binding:        {cl}
+  scroll lock binding:      {sl}
   require user PIN for OTP: {otp}"#,
-      nl = format_option(config.numlock),
-      cl = format_option(config.capslock),
-      sl = format_option(config.scrollock),
+      nl = format_option(config.num_lock),
+      cl = format_option(config.caps_lock),
+      sl = format_option(config.scroll_lock),
       otp = config.user_password,
     )?;
     Ok(())
@@ -629,12 +681,13 @@ pub fn config_get(ctx: &mut Context<'_>) -> anyhow::Result<()> {
 
 /// Write the Nitrokey configuration.
 pub fn config_set(ctx: &mut Context<'_>, args: args::ConfigSetArgs) -> anyhow::Result<()> {
-  let numlock = args::ConfigOption::try_from(args.no_numlock, args.numlock, "numlock")
-    .context("Failed to apply numlock configuration")?;
-  let capslock = args::ConfigOption::try_from(args.no_capslock, args.capslock, "capslock")
-    .context("Failed to apply capslock configuration")?;
-  let scrollock = args::ConfigOption::try_from(args.no_scrollock, args.scrollock, "scrollock")
-    .context("Failed to apply scrollock configuration")?;
+  let num_lock = args::ConfigOption::try_from(args.no_num_lock, args.num_lock, "numlock")
+    .context("Failed to apply num lock configuration")?;
+  let caps_lock = args::ConfigOption::try_from(args.no_caps_lock, args.caps_lock, "capslock")
+    .context("Failed to apply caps lock configuration")?;
+  let scroll_lock =
+    args::ConfigOption::try_from(args.no_scroll_lock, args.scroll_lock, "scrollock")
+      .context("Failed to apply scroll lock configuration")?;
   let otp_pin = if args.otp_pin {
     Some(true)
   } else if args.no_otp_pin {
@@ -649,9 +702,9 @@ pub fn config_set(ctx: &mut Context<'_>, args: args::ConfigSetArgs) -> anyhow::R
       .get_config()
       .context("Failed to get current configuration")?;
     let config = nitrokey::Config {
-      numlock: numlock.or(config.numlock),
-      capslock: capslock.or(config.capslock),
-      scrollock: scrollock.or(config.scrollock),
+      num_lock: num_lock.or(config.num_lock),
+      caps_lock: caps_lock.or(config.caps_lock),
+      scroll_lock: scroll_lock.or(config.scroll_lock),
       user_password: otp_pin.unwrap_or(config.user_password),
     };
     device
@@ -1037,6 +1090,140 @@ pub fn pws_status(ctx: &mut Context<'_>, all: bool) -> anyhow::Result<()> {
   })
 }
 
+/// Find and list all available extensions.
+///
+/// The logic used in this function should use the same criteria as
+/// `resolve_extension`.
+pub(crate) fn discover_extensions(path_var: &ffi::OsStr) -> anyhow::Result<Vec<String>> {
+  let dirs = env::split_paths(path_var);
+  let mut commands = Vec::new();
+
+  for dir in dirs {
+    match fs::read_dir(&dir) {
+      Ok(entries) => {
+        for entry in entries {
+          let entry = entry?;
+          let path = entry.path();
+          if path.is_file() {
+            let name = entry.file_name();
+            let file = name.to_string_lossy();
+            if file.starts_with(NITROCLI_EXT_PREFIX) {
+              let mut file = file.into_owned();
+              file.replace_range(..NITROCLI_EXT_PREFIX.len(), "");
+              commands.push(file);
+            }
+          }
+        }
+      }
+      Err(ref err) if err.kind() == io::ErrorKind::NotFound => (),
+      x => x
+        .map(|_| ())
+        .with_context(|| format!("Failed to iterate entries of directory {}", dir.display()))?,
+    }
+  }
+  Ok(commands)
+}
+
+/// Resolve an extension provided by name to an actual path.
+///
+/// Extensions are (executable) files that have the "nitrocli-" prefix
+/// and are discoverable via the `PATH` environment variable.
+///
+/// The logic used in this function should use the same criteria as
+/// `discover_extensions`.
+pub(crate) fn resolve_extension(
+  path_var: &ffi::OsStr,
+  ext_name: &ffi::OsStr,
+) -> anyhow::Result<path::PathBuf> {
+  let mut bin_name = ffi::OsString::from(NITROCLI_EXT_PREFIX);
+  bin_name.push(ext_name);
+
+  for dir in env::split_paths(path_var) {
+    let mut bin_path = dir.clone();
+    bin_path.push(&bin_name);
+    // Note that we deliberately do not check whether the file we found
+    // is executable. If it is not we will just fail later on with a
+    // permission denied error. The reasons for this behavior are two
+    // fold:
+    // 1) Checking whether a file is executable in Rust is painful (as
+    //    of 1.37 there exists the PermissionsExt trait but it is
+    //    available only for Unix based systems).
+    // 2) It is considered a better user experience to resolve to an
+    //    extension even if it later turned out to be not usable over
+    //    not showing it and silently doing nothing -- mostly because
+    //    anything residing in PATH should be executable anyway and
+    //    given that its name also starts with nitrocli- we are pretty
+    //    sure that's a bug on the user's side.
+    if bin_path.is_file() {
+      return Ok(bin_path);
+    }
+  }
+
+  let err = if let Some(name) = bin_name.to_str() {
+    format!("Extension {} not found", name).into()
+  } else {
+    borrow::Cow::from("Extension not found")
+  };
+  Err(io::Error::new(io::ErrorKind::NotFound, err).into())
+}
+
+/// Run an extension.
+pub fn extension(ctx: &mut Context<'_>, args: Vec<ffi::OsString>) -> anyhow::Result<()> {
+  // Note that while `Command` would actually honor PATH by itself, we
+  // do not want that behavior because it would circumvent the execution
+  // context we use for testing. As such, we need to do our own search.
+  let mut args = args.into_iter();
+  let ext_name = args.next().context("No extension specified")?;
+  let path_var = ctx.path.as_ref().context("PATH variable not present")?;
+  let ext_path = resolve_extension(&path_var, &ext_name)?;
+
+  // Note that theoretically we could just exec the extension and be
+  // done. However, the problem with that approach is that it makes
+  // testing extension support much more nasty, because the test process
+  // would be overwritten in the process, requiring us to essentially
+  // fork & exec nitrocli beforehand -- which is much more involved from
+  // a cargo test context.
+  let mut cmd = process::Command::new(&ext_path);
+
+  if let Some(model) = ctx.config.model {
+    let _ = cmd.env(crate::NITROCLI_MODEL, model.to_string());
+  }
+
+  if let Some(usb_path) = &ctx.config.usb_path {
+    let _ = cmd.env(crate::NITROCLI_USB_PATH, usb_path);
+  }
+
+  // TODO: We may want to take this path from the command execution
+  //       context.
+  let binary = env::current_exe().context("Failed to retrieve path to nitrocli binary")?;
+  let serial_numbers = ctx
+    .config
+    .serial_numbers
+    .iter()
+    .map(ToString::to_string)
+    .collect::<Vec<_>>()
+    .join(",");
+
+  let out = cmd
+    .env(crate::NITROCLI_BINARY, binary)
+    .env(crate::NITROCLI_VERBOSITY, ctx.config.verbosity.to_string())
+    .env(crate::NITROCLI_NO_CACHE, ctx.config.no_cache.to_string())
+    .env(crate::NITROCLI_SERIAL_NUMBERS, serial_numbers)
+    .args(args)
+    .output()
+    .with_context(|| format!("Failed to execute extension {}", ext_path.display()))?;
+  ctx.stdout.write_all(&out.stdout)?;
+  ctx.stderr.write_all(&out.stderr)?;
+
+  if out.status.success() {
+    Ok(())
+  } else if let Some(rc) = out.status.code() {
+    Err(anyhow::Error::new(crate::DirectExitError(rc)))
+  } else {
+    Err(anyhow::Error::new(crate::DirectExitError(1)))
+  }
+}
+
 #[cfg(test)]
 mod tests {
   use super::*;
