@@ -9,6 +9,8 @@ use nitrokey_test::test as test_device;
 
 mod config;
 mod encrypted;
+mod extensions;
+mod fill;
 mod hidden;
 mod list;
 mod lock;
@@ -22,6 +24,7 @@ mod unencrypted;
 
 struct Nitrocli {
   model: Option<nitrokey::Model>,
+  path: Option<ffi::OsString>,
   admin_pin: Option<ffi::OsString>,
   user_pin: Option<ffi::OsString>,
   new_admin_pin: Option<ffi::OsString>,
@@ -33,6 +36,7 @@ impl Nitrocli {
   pub fn new() -> Self {
     Self {
       model: None,
+      path: None,
       admin_pin: Some(nitrokey::DEFAULT_ADMIN_PIN.into()),
       user_pin: Some(nitrokey::DEFAULT_USER_PIN.into()),
       new_admin_pin: None,
@@ -53,6 +57,12 @@ impl Nitrocli {
     self
   }
 
+  /// Set the `PATH` used for looking up extensions.
+  fn path(mut self, path: impl Into<ffi::OsString>) -> Self {
+    self.path = Some(path.into());
+    self
+  }
+
   pub fn admin_pin(mut self, pin: impl Into<ffi::OsString>) -> Self {
     self.admin_pin = Some(pin.into());
     self
@@ -75,8 +85,10 @@ impl Nitrocli {
 
   fn model_to_arg(model: nitrokey::Model) -> &'static str {
     match model {
+      nitrokey::Model::Librem => "--model=librem",
       nitrokey::Model::Pro => "--model=pro",
       nitrokey::Model::Storage => "--model=storage",
+      _ => panic!("Unexpected model in test suite: {}", model),
     }
   }
 
@@ -98,6 +110,8 @@ impl Nitrocli {
     let ctx = &mut crate::Context {
       stdout: &mut stdout,
       stderr: &mut stderr,
+      is_tty: false,
+      path: self.path.clone(),
       admin_pin: self.admin_pin.clone(),
       user_pin: self.user_pin.clone(),
       new_admin_pin: self.new_admin_pin.clone(),
